;
; hello.stepper.44.full.asm
;
; stepper motor full step pulse sequence
;
; Neil Gershenfeld
; CBA MIT 11/14/08
;
; (c) Massachusetts Institute of Technology 2008
; Permission granted for experimental and personal use;
; license for commercial sale available from MIT.
;
.include "tn44def.inc"
;
; definitions
;
.equ serial_comm = PB2 ; 
.equ brown = PA0 ; MOSFET output pins
.equ black = PA1 ; "
.equ yellow = PA2 ; "
.equ orange = PA3 ; "
.equ on_count = 40 ; PWM on count
.equ off_count = 10 ; PWM off count
.equ pulse_count = 200 ; PWM cycles
.equ rxpin = PB2 ; receive pin
.equ comm_pins = PINA; comm pins - I DON'T THINK THIS IS RIGHT, BUT I DON'T KNOW

.equ serial_clockwise = 1 ;
.equ serial_counterclockwise = 2 ;
;
; registers
;
.def temp = R16
.def temp1 = R17
.def serial_temp_register = R18
.def serial_comm_register = R19
.def bitcnt = R20; bit counter
.def rxbyte = R21; receive byte
;
; code segment
;
.cseg
.org 0
rjmp reset

;
; half_bit_delay
; serial half bit delay
;
half_bit_delay:
   ldi temp, 25; 115200 baud (20 MHz clock /1)
   half_bit_delay_loop:
      dec temp
      brne half_bit_delay_loop
   ret

;
; getchar
; assumes no line driver (doesn't invert bits)
;
getchar:
   ldi bitcnt, 9; 8 data + 1 stop bit
   getchar1:
      sbis comm_pins, rxpin; wait for start bit - TODO: define comm_pins
      rjmp getchar1
   rcall half_bit_delay; delay to middle of bit
   getchar2:
      rcall half_bit_delay; bit delay
      rcall half_bit_delay; "
      clc; clear carry
      sbis comm_pins, rxpin; if RX pin high skip
         sec; otherwise set carry
      dec bitcnt
      breq getchar3; return if all bits read
      ror rxbyte; otherwise shift bit into receive byte
      rjmp getchar2; go get next bit
   getchar3:
      ret

;
; yellow, brown PWM pulse
;
pulse_yellow_brown:
   ldi temp, pulse_count
   loop_yellow_brown:
      ldi temp1, on_count
      sbi PORTA, yellow
      sbi PORTA, brown
      on_yellow_brown:
         dec temp1
         brne on_yellow_brown
      ldi temp1, off_count
      cbi PORTA, yellow
      cbi PORTA, brown
      off_yellow_brown:
         dec temp1
         brne off_yellow_brown
      dec temp
      brne loop_yellow_brown
   ret
;
; yellow, black PWM pulse
;
pulse_yellow_black:
   ldi temp, pulse_count
   loop_yellow_black:
      ldi temp1, on_count
      sbi PORTA, yellow 
      sbi PORTA, black
      on_yellow_black:
         dec temp1
         brne on_yellow_black
      ldi temp1, off_count
      cbi PORTA, yellow
      cbi PORTA, black
      off_yellow_black:
         dec temp1
         brne off_yellow_black
      dec temp
      brne loop_yellow_black
   ret
;
; brown PWM pulse
;
pulse_orange_brown:
   ldi temp, pulse_count
   loop_orange_brown:
      ldi temp1, on_count
      sbi PORTA, orange
      sbi PORTA, brown
      on_orange_brown:
         dec temp1
         brne on_orange_brown
      ldi temp1, off_count
      cbi PORTA, orange
      cbi PORTA, brown
      off_orange_brown:
         dec temp1
         brne off_orange_brown
      dec temp
      brne loop_orange_brown
   ret
;
; orange, black PWM pulse
;
pulse_orange_black:
   ldi temp, pulse_count
   loop_orange_black:
         ldi temp1, on_count
         sbi PORTA, orange
         sbi PORTA, black
         on_orange_black:
            dec temp1
            brne on_orange_black
         ldi temp1, off_count
         cbi PORTA, orange
         cbi PORTA, black
         off_orange_black:
            dec temp1
            brne off_orange_black
      dec temp
      brne loop_orange_black
   ret
;
; clockwise step
;
step_cw:
   rcall pulse_yellow_brown
   rcall pulse_yellow_black
   rcall pulse_orange_black
   rcall pulse_orange_brown
   ret
;
; counter-clockwise step
;
step_ccw:
   rcall pulse_orange_brown
   rcall pulse_orange_black
   rcall pulse_yellow_black
   rcall pulse_yellow_brown
   ret
;
; main program
;
reset:
   ;
   ; set clock divider to /1
   ;
   ldi temp, (1 << CLKPCE)
   ldi temp1, (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0)
   out CLKPR, temp
   out CLKPR, temp1
   ;
   ; set stack pointer to top of RAM
   ;
   ldi temp, high(RAMEND)
   out SPH, temp
   ldi temp, low(RAMEND)
   out SPL, temp
   ;
   ; set MOSFET pins
   ;
   cbi PORTA, brown
   sbi DDRA, brown
   cbi PORTA, black
   sbi DDRA, black
   cbi PORTA, yellow
   sbi DDRA, yellow
   cbi PORTA, orange
   sbi DDRA, orange
   ;
   ; main loop
   ;
   loop:
   	
      rcall getchar ; Get the next character
      
      ; Check if this is a counterclockwise command
      ldi serial_temp_register, serial_counterclockwise
      cp rxbyte, serial_temp_register
      breq serial_ccw
      
      ; Check if this is a clockwise command
      ldi serial_temp_register, serial_clockwise
      cp rxbyte, serial_temp_register
      breq serial_cw
      ;rjmp loop  ; TODO - This is right, but need to find correct ASCII values
	
      serial_ccw:
      	rcall step_ccw
	rjmp loop
	
      serial_cw:
      	rcall step_cw
	rjmp loop
